//
//  LineNumberPlugin.m
//  linenumber_plugin
//
//  Created by Henning Kiel on 29.04.09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "TestSuggestionsPlugin.h"
#import "JRSwizzle.h"
#import "TextMateExtensions.h"
#import "RelevantTestcases.h"
#import "RelevantTestcasesController.h"

TestSuggestionsPlugin *hennkDelegate;


@interface TestSuggestionsPlugin(private)
- (void)installMenuItem;
- (void)uninstallMenuItem;
- (void)loadAvailableCoverageData;
@end

@implementation TestSuggestionsPlugin

- (id)initWithPlugInController:(id <TMPlugInController>)aController
{
	self = [self init];
  
  hennkDelegate = self;
  
  relevantTestCases = [[RelevantTestcases alloc] init];
  
  NSLog(@"Trying swizzle...");
  NSError *error = nil;
  [OakStatusBar jr_swizzleMethod:@selector(setLineNumber:) withMethod:@selector(hennkSetLineNumber:) error:&error];
  if (error != nil)
    NSLog(@"Swizzle unsuccesful: %@", [error localizedDescription]);
  
  [NSWindowPoser poseAsClass:[NSWindow class]];
  
  [self installMenuItem];
  
	return self;
}

- (void)dealloc;
{
  [relevantTestCasesController release];
  [relevantTestCases release];
	[self uninstallMenuItem];
	[super dealloc];
}

- (void)loadAvailableCoverageData;
{
  NSString *dataFilesDir = @"/Users/hkiel/Desktop/Diplomarbeit/Experiment_1/textmate_plugin/data";
  NSArray *possibleDataFiles = [[NSFileManager defaultManager] contentsOfDirectoryAtPath:dataFilesDir error:NULL];
  NSEnumerator *data_files_enum = [possibleDataFiles objectEnumerator];
  NSString *data_file_path;
  dataFilesMenu = [[NSMenu alloc] initWithTitle:@"Coverage Data"];
  while ((data_file_path = [data_files_enum nextObject]))
  {
    if (![[data_file_path pathExtension] isEqualToString:@"plist"])
      continue;
    
    NSMenuItem *tempItem = [[[NSMenuItem alloc] initWithTitle:[dataFilesDir stringByAppendingPathComponent:data_file_path] action:@selector(loadCoverageDataFor:) keyEquivalent:@""] autorelease];
    [tempItem setTarget:relevantTestCasesController];
    [tempItem setEnabled:YES];
    [dataFilesMenu addItem:tempItem];
  }
  [coverageDataMenuItem setEnabled:YES];
  [windowMenu setSubmenu:dataFilesMenu forItem:coverageDataMenuItem];
}

- (void)installMenuItem;
{
	if(windowMenu = [[[[NSApp mainMenu] itemWithTitle:@"Window"] submenu] retain])
	{
		unsigned index = [[windowMenu itemArray] count];
    
		showTestCaseSuggestionsMenuItem = [[NSMenuItem alloc] initWithTitle:@"Show Test Case Suggestions" action:@selector(showSuggestions:) keyEquivalent:@""];
		[showTestCaseSuggestionsMenuItem setTarget:self];
		[windowMenu insertItem:showTestCaseSuggestionsMenuItem atIndex:index];
    index++;

    coverageDataMenuItem = [[NSMenuItem alloc] initWithTitle:@"Available Coverage Data" action:NULL keyEquivalent:@""];
		[windowMenu insertItem:coverageDataMenuItem atIndex:index];
  }
}

- (void)uninstallMenuItem;
{
	[windowMenu removeItem:showTestCaseSuggestionsMenuItem];
  
	[showTestCaseSuggestionsMenuItem release];
	showTestCaseSuggestionsMenuItem = nil;
  
	[windowMenu release];
	windowMenu = nil;
  
  [dataFilesMenu release];
  dataFilesMenu = nil;
}

- (void)didChangeFileTo:(NSString *)newFileName;
{
  NSLog(@"%s %@", _cmd, newFileName);
  [relevantTestCases setSourceFilePath:newFileName];
}

- (void)didChangeLineNumberTo:(NSNumber *)newLineNumber;
{
//  NSLog(@"hennkSetLineNumber called with %@", newLineNumber);
  [relevantTestCases setLineNumber:newLineNumber];
}

- (IBAction)showSuggestions:(id)sender;
{
  if (relevantTestCasesController != nil)
    return;
  
  relevantTestCasesController = [[RelevantTestcasesController alloc] initWithWindowNibName:@"RelevantTestcasesWindow"];
  [relevantTestCasesController showWindow:self];
  [relevantTestCasesController setRelevantTestcases:relevantTestCases];

  [self loadAvailableCoverageData];
}
@end
  
